SystemOrganization addCategory: #'Multilingual-Tests-TextConversion'!

TestCase subclass: #ByteTextConverterTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Multilingual-Tests-TextConversion'!

!ByteTextConverterTest methodsFor: 'testing' stamp: 'nice 7/26/2009 22:44'!
testConversionToFrom
	"Non regresson test for http://code.google.com/p/pharo/issues/detail?id=986"

	self assert: (('' convertToEncoding: 'mac-roman') convertFromEncoding: 'mac-roman') = ''! !

!ByteTextConverterTest methodsFor: 'testing' stamp: 'HenrikSperreJohansen 6/25/2012 12:43'!
testLatin2ToUnicodeConversion
        | latin2Characters utfCodePoints decodingStream unicodeString |

        latin2Characters := #[16rBE 16rFD 16rE1 16rC8] asString.
	utfCodePoints := #(16r017E 16rFD 16rE1 16r010C).
	
	"Converters assume that characters you want to put/get from a stream are Unicode characters"
	"So our source is a string of latin 2"
	decodingStream := (MultiByteBinaryOrTextStream 
                with:  latin2Characters 
                encoding: 'ISO-8859-2') reset.
	unicodeString := decodingStream contents.
	"our sourcelatin2 string contains characters outside latin1"
	self assert: unicodeString isWideString.
	self assert: (unicodeString at: 1) charCode = (utfCodePoints at: 1) .
	self assert: (unicodeString at: 2) charCode = (utfCodePoints at: 2) .
	self assert: (unicodeString at: 3) charCode = (utfCodePoints at: 3) .
	self assert: (unicodeString at: 4) charCode = (utfCodePoints at: 4) .! !

!ByteTextConverterTest methodsFor: 'testing' stamp: 'HenrikSperreJohansen 6/25/2012 12:55'!
testUnicodeToLatin2Conversion
        | latin2Bytes internalString encodingStream encodedBytes  |

        latin2Bytes := #[16rBE 16rFD 16rE1 16rC8] .
	internalString := String streamContents: [:s | 
		s nextPut: 16r017E asCharacter;
		   nextPut: 16rFD asCharacter;
		  nextPut: 16rE1 asCharacter;
		  nextPut: 16r010C asCharacter].
	"Converters assume that characters you want to put/get from a stream are Unicode characters"
	"So we use a stream with an encoder, then put the internalString to it"
	encodingStream := (MultiByteBinaryOrTextStream 
                with:  String new 
                encoding: 'ISO-8859-2') reset.
	encodingStream nextPutAll: internalString.
	"the encoded string in the stream does not contain any characters > 255"
	encodedBytes := encodingStream binary reset contents.
	
	
	self assert: (encodedBytes at: 1)  = (latin2Bytes at: 1) .
	self assert: (encodedBytes at: 2)  = (latin2Bytes at: 2) .
	self assert: (encodedBytes at: 3)  = (latin2Bytes at: 3) .
	self assert: (encodedBytes at: 4)  = (latin2Bytes at: 4) .! !

TestCase subclass: #EUCTextConverterTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Multilingual-Tests-TextConversion'!

!EUCTextConverterTest methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 2/27/2013 20:57'!
testEUCJP

	| encodingName halfwidthKatakanaLetterKa cjkUnifiedIdeograph6CB4 cjkUnifiedIdeograph50BA
	  encodedHalfwidthKatakanaLetterKa encodedCJKUnifiedIdeograph6CB4 encodedCJKUnifiedIdeograph50BA
	  japaneseInJapanese |

	encodingName := 'euc-jp'.

	"Example characters taken from: http://www.sljfaq.org/afaq/encodings.html#encodings-EUC-JP"
	halfwidthKatakanaLetterKa := Character codePoint: 16rFF76.
	cjkUnifiedIdeograph6CB4 := Character codePoint: 16r6CB3.
	cjkUnifiedIdeograph50BA := Character codePoint: 16r50BA.
	
	encodedHalfwidthKatakanaLetterKa := #[16r8E 16rB6].
	encodedCJKUnifiedIdeograph6CB4 := #[16rB2 16rCF].
	encodedCJKUnifiedIdeograph50BA := #[16r8F 16rB2 16rB0].

	self assert: ((String with: cjkUnifiedIdeograph6CB4) convertToEncoding: encodingName) asByteArray = encodedCJKUnifiedIdeograph6CB4.
	"EUCJPTextConverter does not currently support encoding the characters defined in the JIS-X-0201 and JIS-X-0212 standards:"
	self assert: ((String with: halfwidthKatakanaLetterKa) convertToEncoding: encodingName) asByteArray = #[].
	self assert: ((String with: cjkUnifiedIdeograph50BA) convertToEncoding: encodingName) asByteArray = #[].

	self assert: (encodedCJKUnifiedIdeograph6CB4 asString convertFromEncoding: encodingName) = (String with: cjkUnifiedIdeograph6CB4).
	"EUCJPTextConverter does not currently support decoding the representations that start with the 8E and 8F markers:"
	self assert: (encodedHalfwidthKatakanaLetterKa asString convertFromEncoding: encodingName) = (String new).
	self assert: (encodedCJKUnifiedIdeograph50BA asString convertFromEncoding: encodingName) = (String new).


	japaneseInJapanese := #(16r65E5 16r672C 16r8A9E) collect: [ :codePoint | Character codePoint: codePoint ] as: String.
	((japaneseInJapanese convertToEncoding: encodingName) convertFromEncoding: encodingName) = japaneseInJapanese.! !

TestCase subclass: #MultiByteFileStreamTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Multilingual-Tests-TextConversion'!

!MultiByteFileStreamTest methodsFor: 'support' stamp: 'HenrikSperreJohansen 5/19/2011 09:53'!
chunkString
  "A chunk larger than buffer (currently 2048), but not long enough to fill it completely again. Therefore, buffer should contain some elements at end that are not part of the real buffer"
  | smallChunk chunkString |
  chunkString := 
  String new: (MultiByteFileStream new ascii enableReadBuffering braceArray size * 1.3) floor.
  smallChunk := 'ASDFASDFASDFASDFasdfasdfasdfasdfQWERQWERQWERqwerqwerqwer!! !!'.
  1 to: chunkString size by: smallChunk size do: [:ix |
    chunkString replaceFrom: ix to: (ix + smallChunk size -1 min: chunkString size) with: smallChunk startingAt: 1].
  ^chunkString! !

!MultiByteFileStreamTest methodsFor: 'support' stamp: 'HenrikSperreJohansen 5/19/2011 09:53'!
lineEndTestFile
	^'lineEndTesting.txt'! !

!MultiByteFileStreamTest methodsFor: 'testing' stamp: 'SeanDeNigris 7/12/2012 08:44'!
tearDown
	'foobug6933' asFileReference delete.
	self lineEndTestFile asFileReference delete.! !

!MultiByteFileStreamTest methodsFor: 'testing' stamp: 'SeanDeNigris 7/12/2012 08:44'!
testBasicChunk
	 | internalStream chunkKey |
	internalStream := self chunkString readStream.
	chunkKey := OrderedCollection new.
	[internalStream atEnd] whileFalse: 
		[ | chunk |
		chunk := internalStream nextChunk.
		chunkKey add: {internalStream position. chunk size. chunk}].
	[self writeChunkToFile.
	FileStream readOnlyFileNamed: 'chunkTest.txt' do: [:fileStream |
		1 to: chunkKey size do: [:ix | |chunk|
			chunk := fileStream nextChunk.
			self assert: (chunkKey at: ix) first equals: fileStream position.
			self assert: (chunkKey at: ix) second equals: chunk size.
			self assert: (chunkKey at: ix) last equals: chunk.]].
	] ensure: [
		 'chunkTest.txt' asFileReference delete]! !

!MultiByteFileStreamTest methodsFor: 'testing' stamp: 'SeanDeNigris 7/12/2012 08:44'!
testBinaryUpTo
	"This is a non regression test for bug http://bugs.squeak.org/view.php?id=6933"
	"self run: #testBinaryUpTo"
	
	| foo fileName |
	
	fileName := 'foobug6933'. 
	MultiByteFileStream 
		forceNewFileNamed: fileName 
		do: [ :stream | stream binary.
				stream nextPutAll: #[1 2 3 4] ].

	foo := MultiByteFileStream oldFileNamed: fileName.
	[foo binary.
		self assert: (foo upTo: 3) = #[1 2] 
	] ensure: [
		foo close.
 		fileName  asFileReference delete]. 
    ! !

!MultiByteFileStreamTest methodsFor: 'testing' stamp: 'HenrikSperreJohansen 5/19/2011 10:16'!
testLineEndingsWith: anEncodingName
	 | byteMap |
	"Make a map of different endings, and the expected byte pattern when three cr's are added using it"
	byteMap := Dictionary new.
	byteMap at: #cr put: #[13 13 13];
			at: #lf put: #[10 10 10];
			at: #crlf put: #[13 10 13 10 13 10].
	byteMap keysDo: [:lineEnding  |
		FileStream forceNewFileNamed: self lineEndTestFile do: [:stream | 
			stream 
				lineEndConvention: lineEnding;
				converter: (TextConverter newForEncoding: anEncodingName).
			"Test 3 cases: Use of nextPut, nextPutAll:, and nextPutAll with WideStrings"
			stream nextPut: Character cr;
					nextPutAll: Character cr asString;
					nextPutAll: Character cr asString asWideString.].
		FileStream oldFileNamed: self lineEndTestFile do: [:stream | | bytes expected| 
			bytes := stream binary; contents.
			self assert: bytes equals: (byteMap at: lineEnding)]]! !

!MultiByteFileStreamTest methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/3/2012 22:11'!
testReadIntoStartingAtCount
	| testString filename buffer |
	testString := 'lve en Franais'.
	filename := 'test-file-' , 99 atRandom printString , '.txt'.
	filename asFileReference ensureDeleted.
	filename asFileReference
		writeStreamDo: [ :stream | 
			stream
				nextPutAll: testString;
				crlf ].
	buffer := String new: testString size.
	[ 
	filename asFileReference
		readStreamDo: [ :stream | 
			self assert: (stream readInto: buffer startingAt: 1 count: testString size) equals: testString size.
			self assert: buffer equals: testString ] ]
		ensure: [ filename asFileReference ensureDeleted ]! !

!MultiByteFileStreamTest methodsFor: 'support' stamp: 'HenrikSperreJohansen 2/9/2010 11:10'!
writeChunkToFile
	FileStream forceNewFileNamed: 'chunkTest.txt' do: [:stream | stream nextPutAll: self chunkString]! !

TestCase subclass: #UTF16TextConverterTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Multilingual-Tests-TextConversion'!

!UTF16TextConverterTest methodsFor: 'helpers' stamp: 'HenrikSperreJohansen 8/9/2010 16:25'!
assert: originalText isTheSameAfterLittleEndianConversion: aBooleanOrNil

	"Test that encoding / decoding a string witll produce the original string for both byte orders"

	| converter bytes decodedText |

	converter := UTF16TextConverter new.
	aBooleanOrNil ifNotNil: [converter useLittleEndian: aBooleanOrNil].
	bytes := (originalText convertToWithConverter: converter).
	decodedText := bytes convertFromWithConverter: converter.
	
	self assert: originalText equals: decodedText.
	! !

!UTF16TextConverterTest methodsFor: 'helpers' stamp: 'HenrikSperreJohansen 8/9/2010 15:58'!
assertBOMIsWrittenWithOrder: anOrder andConversion: aBlock

	|converter stream  BOM  |
	
	converter := UTF16TextConverter new.
	converter useByteOrderMark: true.
	
	anOrder = #bigEndian ifTrue: [converter useLittleEndian: false].
	anOrder = #littleEndian ifTrue: [converter useLittleEndian: true].
	
	BOM := (converter useLittleEndian 
				ifTrue: [16rFFFE]
				ifFalse: [16rFEFF]) asByteArray asString.
	
	stream := String new writeStream. 
	aBlock value: converter value: stream.
	
	self assert: BOM equals: (stream contents  first: 2).
	! !

!UTF16TextConverterTest methodsFor: 'testing' stamp: 'HenrikSperreJohansen 8/9/2010 16:19'!
testBOMWrite
	
	"Test that BOM is written correctly for both endians, both when using a nextPut:, and nextPutAll:"
	
	| batchConversion oneByOneConversion |
	batchConversion := [:converter :stream | converter nextPutAll: 'test' toStream: stream].
	oneByOneConversion := [:converter :stream | 'test' do: [:each | converter nextPut: each toStream: stream]].
	
	self assertBOMIsWrittenWithOrder: #bigEndian 
		andConversion: oneByOneConversion.
	self assertBOMIsWrittenWithOrder: #bigEndian 
		andConversion: batchConversion.
		
	self assertBOMIsWrittenWithOrder: #littleEndian 
		andConversion: oneByOneConversion.
	self assertBOMIsWrittenWithOrder: #littleEndian 
		andConversion: batchConversion.
		
	self assertBOMIsWrittenWithOrder: nil 
		andConversion: oneByOneConversion.
	self assertBOMIsWrittenWithOrder: nil 
		andConversion: batchConversion.! !

!UTF16TextConverterTest methodsFor: 'testing' stamp: 'HenrikSperreJohansen 8/9/2010 16:26'!
testByteOrders

	"Test that encoding / decoding a string witll produce the original string for both byte orders"

	"Default endianness"
	self assert: 'test' isTheSameAfterLittleEndianConversion: nil.
	"Big endian"
	self assert: 'test' isTheSameAfterLittleEndianConversion: false.
	"Little endian"
	self assert: 'test' isTheSameAfterLittleEndianConversion: true.
! !

!UTF16TextConverterTest methodsFor: 'testing' stamp: 'HenrikSperreJohansen 8/9/2010 16:43'!
testNoBOMWriteInMiddleOfFile
	
	"Test that BOM is not written when our first write does not occur at start of writestream"
	
	| converter stream |
	converter := UTF16TextConverter new.

	
	stream := String new writeStream.
	converter nextPutAll: 'testing' toStream: stream.
	stream position: 2.
	converter useByteOrderMark: true.
		
	converter nextPut: $a toStream: stream.
	self assert: 0 equals:  (stream contents indexOfSubCollection: 16rFEFF asByteArray asString)! !

TestCase subclass: #UTF8TextConverterTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Multilingual-Tests-TextConversion'!

!UTF8TextConverterTest methodsFor: 'testing' stamp: 'HenrikSperreJohansen 8/17/2012 11:37'!
testNextPutAllStartingAt
	"Test that non-ascii ByteString characters are converted correctly when using next:putAll:startingAt:"
	|converter stream source|
	converter := UTF8TextConverter new.
	stream := (String new: 10) writeStream.
	source := 'abcdefgh'. "Notice  at 7, which is a non-ascii character in latin1-range, hence ByteString encoded"
converter next: 5 putAll: source startingAt: 5 toStream: stream.
"C3A5 is twobyte utf8-encoding of "
self assert: stream contents asByteArray = #[16r65 16r66 16rC3 16rA5 16r67 16r68] ! !

!UTF8TextConverterTest methodsFor: 'testing' stamp: 'HenrikSperreJohansen 6/29/2012 10:53'!
testPutSingle2ByteEncodedCharacter
	| actual |
	"Test that the utf8-encoded bytes are stored when stream is ascii.
	(In the case of internal writeStreams, that its destination is a String)"
	actual := String streamContents: [ :stream |
		| converter |
		converter := UTF8TextConverter new.
		converter
			nextPut: $
			toStream: stream ].
	self assert: actual asByteArray = #[2r11000011  2r10100111]
	! !

!UTF8TextConverterTest methodsFor: 'testing' stamp: 'HenrikSperreJohansen 6/29/2012 10:53'!
testPutSingle3ByteEncodedCharacter
	| actual |
	"Test that the utf8-encoded bytes are stored when stream is ascii.
	(In the case of internal writeStreams, that its destination is a String)"
	actual := String streamContents: [ :stream |
		| converter |
		converter := UTF8TextConverter new.
		converter
			nextPut: (Unicode value: 16r20AC)
			toStream: stream ].
	self assert: actual asByteArray = #[2r11100010 2r10000010 2r10101100]
	! !

!UTF8TextConverterTest methodsFor: 'testing' stamp: 'HenrikSperreJohansen 6/29/2012 10:52'!
testPutSingle4ByteEncodedCharacter
	| actual |
	"Test that the utf8-encoded bytes are stored when stream is ascii.
	(In the case of internal writeStreams, that its destination is a String)"
	actual := String streamContents: [ :stream |
		| converter |
		converter := UTF8TextConverter new.
		converter
			nextPut: (Unicode value: 16r24B62)
			toStream: stream ].
	self assert: actual asByteArray = #[2r11110000 2r10100100 2r10101101 2r10100010]
	! !

!UTF8TextConverterTest methodsFor: 'testing' stamp: 'HenrikSperreJohansen 7/17/2012 13:07'!
testPutSingleCharacterBinary
	| actual |
	"Test that the raw 8/32bit representations of Characters/WideCharacters are stored when stream is binary.
	(In the case of internal writeStreams, that its destination is a ByteArray)"
	actual := ByteArray streamContents: [ :stream |
		| converter |
		converter := UTF8TextConverter new.
		converter
			nextPut: $a
			toStream: stream.
		converter
			nextPut: (Unicode value: 16r20AC)
			toStream: stream ].
	self assert: actual = #[97 0 0 16r20 16rAC]! !
